@@ -110,16 +110,16 @@
 (defconst remind-keywords
   (sort
    (list "ADDOMIT" "AFTER" "AT" "BAN" "BANNER" "BEFORE" "CAL" "CLEAR"
-         "CLEAR-OMIT-CONTEXT" "DEBUG" "DO" "DUMP" "DUMPVARS"
+         "CLEAR-OMIT-CONTEXT" "COMPLETE-THROUGH" "DEBUG" "DO" "DUMP" "DUMPVARS"
          "DURATION" "ELSE" "ENDIF" "ERRMSG" "EXIT" "EXPR" "FIRST"
          "FLUSH" "FOURTH" "FRENAME" "FROM" "FSET" "FUNSET" "IF"
          "IFTRIG" "IN" "INC" "INCLUDE" "INCLUDECMD" "INFO" "LAST"
-         "LASTDAY" "LASTWORKDAY" "MAYBE" "MAYBE-UNCOMPUTABLE" "MSF"
+         "LASTDAY" "LASTWORKDAY" "MAX-OVERDUE" "MAYBE" "MAYBE-UNCOMPUTABLE" "MSF"
          "MSG" "NOQUEUE" "OMIT" "OMITFUNC" "ONCE" "POP"
          "POP-OMIT-CONTEXT" "POP-FUNCS" "POP-VARS" "PRESERVE" "PRIORITY" "PS"
-         "PSFILE" "PUSH" "PUSH-FUNCS" "PUSH-VARS" "PUSH-OMIT-CONTEXT" "REM"
+         "PSFILE" "PUSH" "PUSH-FUNCS" "PUSH-VARS" "PUSH-OMIT-CONTEXT" "REM" "RETURN"
          "RUN" "SATISFY" "SCAN" "SCANFROM" "SCHED" "SECOND" "SET"
-         "SKIP" "SPECIAL" "SYSINCLUDE" "TAG" "THIRD" "THROUGH"
+         "SKIP" "SPECIAL" "SYSINCLUDE" "TAG" "THIRD" "THROUGH" "TODO"
          "TRANSLATE" "TRANS" "UNSET" "UNTIL" "WARN")
    #'(lambda (a b) (> (length a) (length b)))))
 
@@ -137,8 +137,8 @@
          "$DefaultPrio" "$DefaultTDelta" "$DeltaOverride"
          "$DontFork" "$DontQueue" "$DontTrigAts" "$EndSent" "$EndSentIg"
          "$ExpressionTimeLimit" "$February" "$FirstIndent" "$FoldYear"
-         "$FormWidth" "$Friday" "$Fromnow" "$Hour" "$Hplu" "$HushMode"
-         "$IgnoreOnce" "$InfDelta" "$IntMax" "$IntMin" "$Is" "$January" "$July"
+         "$FormWidth" "$Friday" "$Fromnow" "$HideCompletedTodos" "$Hour" "$Hplu" "$HushMode"
+         "$IgnoreOnce" "$InfDelta" "$IntMax" "$IntMin" "$Is" "$January" "$JSONMode" "$July"
          "$June" "$LatDeg" "$Latitude" "$LatMin" "$LatSec" "$Location"
          "$LongDeg" "$Longitude" "$LongMin" "$LongSec" "$March" "$MaxFullOmits"
          "$MaxLateMinutes" "$MaxPartialOmits" "$MaxSatIter" "$MaxStringLen"
@@ -148,7 +148,7 @@
          "$PrefixLineNo" "$PSCal" "$RunOff" "$Saturday" "$September"
          "$SimpleCal" "$SortByDate" "$SortByPrio" "$SortByTime" "$SubsIndent"
          "$Sunday" "$SuppressImplicitWarnings" "$SuppressLRM" "$SysInclude" "$T" "$Tb" "$Td"
-         "$TerminalBackground" "$Thursday" "$TimeSep" "$TimetIs64bit" "$Tm" "$Today"
+         "$TerminalBackground" "$Thursday" "$TimeSep" "$TimetIs64bit" "$Tm" "$Today" "$TodoFilter"
          "$Tomorrow" "$Tt" "$Tuesday" "$Tw" "$Ty" "$U" "$Ud" "$Um"
          "$UntimedFirst" "$Use256Colors" "$UseBGVTColors" "$UseTrueColors"
          "$UseVTColors" "$Uw" "$Uy" "$Was" "$Wednesday")
@@ -178,9 +178,9 @@
          "ostype" "pad" "plural" "psmoon" "psshade" "realcurrent" "realnow"
          "realtoday" "rows" "sgn" "shell" "shellescape" "slide" "soleq"
          "stdout" "strlen" "substr" "sunrise" "sunset" "time" "timepart"
-         "timezone" "today" "trig" "trigback" "trigbase" "trigdate" "trigdatetime"
+         "timezone" "today" "trig" "trigback" "trigbase" "trigcompletethrough" "trigdate" "trigdatetime"
          "trigdelta" "trigduration" "trigeventduration" "trigeventstart"
-         "trigfrom" "trigger" "triginfo" "trigpriority" "trigrep"
+         "trigfrom" "trigger" "triginfo" "trigistodo" "trigmaxoverdue" "trigpriority" "trigrep"
          "trigscanfrom" "trigtags" "trigtime" "trigtimedelta" "trigtimerep"
          "triguntil" "trigvalid" "typeof" "tzconvert" "upper" "utctolocal"
          "value" "version" "weekno" "wkday" "wkdaynum" "year"
