@@ -32,6 +32,80 @@ static int ParseUntil (ParsePtr s, Trigger *t, int type);
 static int ShouldTriggerBasedOnWarn (Trigger const *t, int dse, int *err);
 static int ComputeTrigDuration(TimeTrig const *t);
 
+void remove_trailing_newlines(DynamicBuffer *buf)
+{
+    char *s = (char *) DBufValue(buf) + DBufLen(buf) - 1;
+    while (s >= DBufValue(buf)) {
+        if (*s == '\n') {
+            *s = 0;
+            s--;
+        } else {
+            break;
+        }
+    }
+}
+static int todo_filtered(Trigger const *t)
+{
+    if (t->is_todo && TodoFilter == ONLY_EVENTS) return 1;
+    if (!t->is_todo && TodoFilter == ONLY_TODOS) return 1;
+    return 0;
+}
+
+static int
+get_raw_scanfrom(Trigger const *t)
+{
+    if (t->from != NO_DATE) {
+        if (t->from > DSEToday) return t->from;
+        return DSEToday;
+    }
+    if (t->scanfrom == NO_SCANFROM) return NO_SCANFROM;
+    if (t->scanfrom > 0) return t->scanfrom;
+
+    /* Relative SCANFROM is negative, so subtract from today() */
+    return DSEToday + t->scanfrom;
+}
+
+int
+get_scanfrom(Trigger const *t)
+{
+    int calmode = (DoSimpleCalendar || DoCalendar) ? 1 : 0;
+
+    /* TODOs are treated just like events in calendar mode */
+    if (!calmode && t->is_todo && t->from != NO_DATE) {
+        if (t->complete_through != NO_DATE) {
+            if (t->complete_through+1 > t->from) {
+                return t->complete_through+1;
+            } else {
+                return t->from;
+            }
+        } else {
+            return t->from;
+        }
+    }
+    if (get_raw_scanfrom(t) != NO_SCANFROM) {
+        if (!calmode && t->complete_through != NO_DATE) {
+            if (t->complete_through+1 > get_raw_scanfrom(t)) {
+                return t->complete_through+1;
+            } else {
+                return get_raw_scanfrom(t);
+            }
+        } else {
+            return get_raw_scanfrom(t);
+        }
+    }
+    if (!calmode) {
+        if (t->complete_through != NO_DATE) {
+            return t->complete_through+1;
+        }
+        if (t->is_todo) {
+            /* TODO with no COMPLETE-THROUGH.
+               Scan from the beginning of time */
+            return 0;
+        }
+    }
+    return DSEToday;
+}
+
 static int
 ensure_expr_references_first_local_arg(expr_node *node)
 {
@@ -213,6 +287,18 @@ int DoRem(ParsePtr p)
         return r;
     }
 
+    if (trig.complete_through != NO_DATE && !trig.is_todo) {
+        PurgeEchoLine("%s\n", CurLine);
+        FreeTrig(&trig);
+        return E_COMPLETE_WITHOUT_TODO;
+    }
+
+    if (trig.max_overdue >= 0 && !trig.is_todo) {
+        PurgeEchoLine("%s\n", CurLine);
+        FreeTrig(&trig);
+        return E_MAX_OVERDUE_WITHOUT_TODO;
+    }
+
     if (trig.typ == NO_TYPE) {
         if (!Hush) {
             PurgeEchoLine("%s\n", "#!P! Cannot parse next line");
@@ -286,7 +372,7 @@ int DoRem(ParsePtr p)
         }
     } else {
         /* Calculate the trigger date */
-        dse = ComputeTrigger(trig.scanfrom, &trig, &tim, &r, 1);
+        dse = ComputeTrigger(get_scanfrom(&trig), &trig, &tim, &r, 1);
         if (r) {
             if (PurgeMode) {
                 if (!Hush) {
@@ -311,7 +397,7 @@ int DoRem(ParsePtr p)
         }
     }
     if (PurgeMode) {
-        if (trig.expired || dse < DSEToday) {
+        if (trig.expired || (!trig.is_todo && dse < DSEToday)) {
             if (p->expr_happened) {
                 if (p->nonconst_expr) {
                     if (!Hush) {
@@ -339,8 +425,11 @@ int DoRem(ParsePtr p)
     if (dse == DSEToday &&
         !(!IgnoreOnce &&
           trig.once != NO_ONCE &&
-          GetOnceDate() == DSEToday))
-        QueueReminder(p, &trig, &tim, trig.sched);
+          GetOnceDate() == DSEToday)) {
+        if (!todo_filtered(&trig)) {
+            QueueReminder(p, &trig, &tim, trig.sched);
+        }
+    }
     /* If we're in daemon mode, do nothing over here */
     if (Daemon) {
         FreeTrig(&trig);
@@ -349,7 +438,72 @@ int DoRem(ParsePtr p)
 
     r = OK;
     if (ShouldTriggerReminder(&trig, &tim, dse, &err)) {
-        if ( (r=TriggerReminder(p, &trig, &tim, dse, 0, NULL)) ) {
+        /* Filter unwanted events/todos */
+        if (todo_filtered(&trig)) {
+            FreeTrig(&trig);
+            return OK;
+        }
+
+        if (JSONMode) {
+            DynamicBuffer body;
+            int y, m, d;
+            int if_depth = get_if_pointer() - get_base_if_pointer();
+            DBufInit(&body);
+            int red=-1, green=-1, blue=-1;
+            r=TriggerReminder(p, &trig, &tim, dse, 0, &body, &red, &green, &blue);
+            if (r) {
+                FreeTrig(&trig);
+                return r;
+            }
+            /* Remove trailing newlines from body */
+            remove_trailing_newlines(&body);
+
+            if (!*DBufValue(&body)) {
+                FreeTrig(&trig);
+                return r;
+            }
+            if (JSONLinesEmitted) {
+                printf("},\n");
+            }
+            JSONLinesEmitted++;
+            FromDSE(dse, &y, &m, &d);
+            printf("{\"date\":\"%04d-%02d-%02d\",", y, m+1, d);
+            PrintJSONKeyPairString("filename", GetCurrentFilename());
+            PrintJSONKeyPairInt("lineno", LineNo);
+            if (LineNoStart != LineNo) {
+                PrintJSONKeyPairInt("lineno_start", LineNoStart);
+            }
+            PrintJSONKeyPairString("passthru", trig.passthru);
+            if (trig.duration_days) {
+                PrintJSONKeyPairInt("duration", trig.duration_days);
+            }
+            if (tim.ttime != NO_TIME) {
+                PrintJSONKeyPairInt("time", tim.ttime);
+            }
+            if (p->nonconst_expr) {
+                PrintJSONKeyPairInt("nonconst_expr", 1);
+            }
+            if (if_depth) {
+                PrintJSONKeyPairInt("if_depth", if_depth);
+            }
+            if (tim.delta) {
+                PrintJSONKeyPairInt("tdelta", tim.delta);
+            }
+            if (tim.rep) {
+                PrintJSONKeyPairInt("trep", tim.rep);
+            }
+            if (red >= 0 && red <= 255 && green >= 0 && green <= 255 && blue >=0 && blue <= 255) {
+                PrintJSONKeyPairInt("r", red);
+                PrintJSONKeyPairInt("g", green);
+                PrintJSONKeyPairInt("b", blue);
+            }
+
+            WriteJSONTrigger(&trig, 1);
+            printf("\"body\":\"");
+            PrintJSONString(DBufValue(&body));
+            printf("\"");
+        } else {
+            r=TriggerReminder(p, &trig, &tim, dse, 0, NULL, NULL, NULL, NULL);
             FreeTrig(&trig);
             return r;
         }
@@ -369,6 +523,106 @@ int DoRem(ParsePtr p)
     return r;
 }
 
+/***************************************************************/
+/*                                                             */
+/*  GetFullDate - get a full date, either YYYY-MM-DD or        */
+/*                YEAR MON DAY                                 */
+/*                                                             */
+/*  Returns OK on success or an error code on failure.  Sets   */
+/*  *dse on success.                                           */
+/*                                                             */
+/***************************************************************/
+static int GetFullDate(ParsePtr s, char const *prefix, int *dse)
+{
+    Token tok;
+    DynamicBuffer buf;
+    int y = NO_YR, m = NO_MON, d = NO_DAY;
+    int r;
+    *dse = NO_DATE;
+
+    DBufInit(&buf);
+
+    while(1) {
+        r = ParseToken(s, &buf);
+        if (r) return r;
+        FindToken(DBufValue(&buf), &tok);
+        switch(tok.type) {
+        case T_Year:
+            DBufFree(&buf);
+            if (y != NO_YR) {
+                Eprint("%s: %s", prefix, GetErr(E_YR_TWICE));
+                return E_YR_TWICE;
+            }
+            y = tok.val;
+            break;
+
+        case T_Month:
+            DBufFree(&buf);
+            if (m != NO_MON) {
+                Eprint("%s: %s", prefix, GetErr(E_MON_TWICE));
+                return E_MON_TWICE;
+            }
+            m = tok.val;
+            break;
+
+        case T_Day:
+            DBufFree(&buf);
+            if (d != NO_DAY) {
+                Eprint("%s: %s", prefix, GetErr(E_DAY_TWICE));
+                return E_DAY_TWICE;
+            }
+            d = tok.val;
+            break;
+
+        case T_Date:
+            DBufFree(&buf);
+            if (y != NO_YR) {
+                Eprint("%s: %s", prefix, GetErr(E_YR_TWICE));
+                return E_YR_TWICE;
+            }
+            if (m != NO_MON) {
+                Eprint("%s: %s", prefix, GetErr(E_MON_TWICE));
+                return E_MON_TWICE;
+            }
+            if (d != NO_DAY) {
+                Eprint("%s: %s", prefix, GetErr(E_DAY_TWICE));
+                return E_DAY_TWICE;
+            }
+            if (*dse != NO_DATE) {
+                return E_DAY_TWICE;
+            }
+            *dse = tok.val;
+            /* We're done here! */
+            return OK;
+
+        default:
+            if (tok.type == T_Illegal && tok.val < 0) {
+                Eprint("%s: `%s'", GetErr(-tok.val), DBufValue(&buf));
+                DBufFree(&buf);
+                return -tok.val;
+            }
+            if (*dse == NO_DATE && (y == NO_YR || m == NO_MON || d == NO_DAY)) {
+                Eprint("%s: %s", prefix, GetErr(E_INCOMPLETE));
+                DBufFree(&buf);
+                return E_INCOMPLETE;
+            }
+            if (*dse != NO_DATE) {
+                PushToken(DBufValue(&buf), s);
+                DBufFree(&buf);
+                return OK;
+            }
+            if (!DateOK(y, m, d)) {
+                DBufFree(&buf);
+                return E_BAD_DATE;
+            }
+            *dse = DSE(y, m, d);
+            PushToken(DBufValue(&buf), s);
+            DBufFree(&buf);
+            return OK;
+        }
+    }
+}
+
 /***************************************************************/
 /*                                                             */
 /*  ParseRem                                                   */
@@ -383,6 +637,7 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
     DynamicBuffer buf;
     Token tok;
     int y, m, d;
+    int dse;
     int seen_delta = 0;
 
     DBufInit(&buf);
@@ -401,7 +656,7 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
     trig->addomit = 0;
     trig->noqueue = 0;
     trig->typ = NO_TYPE;
-    trig->scanfrom = NO_DATE;
+    trig->scanfrom = NO_SCANFROM;
     trig->from = NO_DATE;
     trig->priority = DefaultPrio;
     trig->sched[0] = 0;
@@ -418,6 +673,9 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
     tim->rep   = NO_REP;
     tim->duration = NO_TIME;
     trig->need_wkday = 0;
+    trig->is_todo = 0;
+    trig->max_overdue = -1;
+    trig->complete_through = NO_DATE;
     trig->adj_for_last = 0;
     trig->infos = NULL;
 
@@ -430,6 +688,11 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
         /* Figure out what we've got */
         FindToken(DBufValue(&buf), &tok);
         switch(tok.type) {
+        case T_Todo:
+            if (trig->is_todo) return E_TODO_TWICE;
+            trig->is_todo = 1;
+            break;
+
         case T_In:
             /* Completely ignored */
             DBufFree(&buf);
@@ -502,6 +765,25 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
             trig->skip = tok.val;
             break;
 
+        case T_MaxOverdue:
+            if (trig->max_overdue >= 0) return E_MAX_OVERDUE_TWICE;
+            DBufFree(&buf);
+            r = ParseToken(s, &buf);
+            if (r) return r;
+            FindToken(DBufValue(&buf), &tok);
+            DBufFree(&buf);
+            if (tok.type == T_Illegal) {
+                return -tok.val;
+            }
+            if (tok.type != T_Day && tok.type != T_Year && tok.type != T_Number) {
+                return E_EXPECTING_NUMBER;
+            }
+            if (tok.val < 0) {
+                return E_2LOW;
+            }
+            trig->max_overdue = tok.val;
+            break;
+
         case T_Priority:
             DBufFree(&buf);
             r=ParsePriority(s, trig);
@@ -556,6 +838,13 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
             if (r) return r;
             break;
 
+        case T_CompleteThrough:
+            if (trig->complete_through != NO_DATE) return E_COMPLETE_THROUGH_TWICE;
+            r = GetFullDate(s, "COMPLETE-THROUGH", &dse);
+            if (r != OK) return r;
+            trig->complete_through = dse;
+            break;
+
         case T_Until:
             DBufFree(&buf);
             r=ParseUntil(s, trig, tok.type);
@@ -767,8 +1056,8 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
             if (trig->until != NO_UNTIL && trig->until < trig->from) {
                 Wprint(tr("Warning: UNTIL/THROUGH date earlier than FROM date"));
             }
-        } else if (trig->scanfrom != NO_DATE) {
-            if (trig->until != NO_UNTIL && trig->until < trig->scanfrom) {
+        } else if (get_raw_scanfrom(trig) != NO_SCANFROM) {
+            if (trig->until != NO_UNTIL && trig->until < get_raw_scanfrom(trig)) {
                 Wprint(tr("Warning: UNTIL/THROUGH date earlier than SCANFROM date"));
             }
         }
@@ -778,11 +1067,6 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
         Wprint(tr("Warning: Useless use of UNTIL with fully-specified date and no *rep"));
     }
 
-    /* Set scanfrom to default if not set explicitly */
-    if (trig->scanfrom == NO_DATE) {
-        trig->scanfrom = DSEToday;
-    }
-
     /* Check that any SCHED / WARN / OMITFUNC functions refer to
        their arguments */
     check_trigger_function(trig->sched, "SCHED");
@@ -884,93 +1168,25 @@ static int ParseLocalOmit(ParsePtr s, Trigger *t)
 /***************************************************************/
 static int ParseUntil(ParsePtr s, Trigger *t, int type)
 {
-    int y = NO_YR,
-        m = NO_MON,
-        d = NO_DAY;
 
     char const *which;
+    int dse;
+
     if (type == T_Until) {
         which = "UNTIL";
     } else {
         which = "THROUGH";
     }
-    Token tok;
-    int r;
-    DynamicBuffer buf;
-    DBufInit(&buf);
 
     if (t->until != NO_UNTIL) return E_UNTIL_TWICE;
 
-    while(1) {
-        r = ParseToken(s, &buf);
-        if (r) return r;
-        FindToken(DBufValue(&buf), &tok);
-        switch(tok.type) {
-        case T_Year:
-            DBufFree(&buf);
-            if (y != NO_YR) {
-                Eprint("%s: %s", which, GetErr(E_YR_TWICE));
-                return E_YR_TWICE;
-            }
-            y = tok.val;
-            break;
-
-        case T_Month:
-            DBufFree(&buf);
-            if (m != NO_MON) {
-                Eprint("%s: %s", which, GetErr(E_MON_TWICE));
-                return E_MON_TWICE;
-            }
-            m = tok.val;
-            break;
-
-        case T_Day:
-            DBufFree(&buf);
-            if (d != NO_DAY) {
-                Eprint("%s: %s", which, GetErr(E_DAY_TWICE));
-                return E_DAY_TWICE;
-            }
-            d = tok.val;
-            break;
-
-        case T_Date:
-            DBufFree(&buf);
-            if (y != NO_YR) {
-                Eprint("%s: %s", which, GetErr(E_YR_TWICE));
-                return E_YR_TWICE;
-            }
-            if (m != NO_MON) {
-                Eprint("%s: %s", which, GetErr(E_MON_TWICE));
-                return E_MON_TWICE;
-            }
-            if (d != NO_DAY) {
-                Eprint("%s: %s", which, GetErr(E_DAY_TWICE));
-                return E_DAY_TWICE;
-            }
-            FromDSE(tok.val, &y, &m, &d);
-            break;
-
-        default:
-            if (tok.type == T_Illegal && tok.val < 0) {
-                Eprint("%s: `%s'", GetErr(-tok.val), DBufValue(&buf));
-                DBufFree(&buf);
-                return -tok.val;
-            }
-            if (y == NO_YR || m == NO_MON || d == NO_DAY) {
-                Eprint("%s: %s", which, GetErr(E_INCOMPLETE));
-                DBufFree(&buf);
-                return E_INCOMPLETE;
-            }
-            if (!DateOK(y, m, d)) {
-                DBufFree(&buf);
-                return E_BAD_DATE;
-            }
-            t->until = DSE(y, m, d);
-            PushToken(DBufValue(&buf), s);
-            DBufFree(&buf);
-            return OK;
-        }
+    int r = GetFullDate(s, which, &dse);
+    if (r != OK) {
+        return r;
     }
+
+    t->until = dse;
+    return OK;
 }
 
 /***************************************************************/
@@ -980,12 +1196,9 @@ static int ParseUntil(ParsePtr s, Trigger *t, int type)
 /***************************************************************/
 static int ParseScanFrom(ParsePtr s, Trigger *t, int type)
 {
-    int y = NO_YR,
-        m = NO_MON,
-        d = NO_DAY;
-
-    Token tok;
     int r;
+    int y = NO_YR, m = NO_MON, d = NO_DAY;
+    Token tok;
     DynamicBuffer buf;
     char const *word;
 
@@ -996,8 +1209,8 @@ static int ParseScanFrom(ParsePtr s, Trigger *t, int type)
         word = "FROM";
     }
 
-    if (t->scanfrom != NO_DATE) return E_SCAN_TWICE;
-
+    if (t->scanfrom != NO_SCANFROM) return E_SCAN_TWICE;
+    if (t->from != NO_DATE) return E_SCAN_TWICE;
     while(1) {
         r = ParseToken(s, &buf);
         if (r) return r;
@@ -1044,8 +1257,12 @@ static int ParseScanFrom(ParsePtr s, Trigger *t, int type)
                 Eprint("%s: %s", word, GetErr(E_DAY_TWICE));
                 return E_DAY_TWICE;
             }
-            FromDSE(tok.val, &y, &m, &d);
-            break;
+            if (type == FROM_TYPE) {
+                t->from = tok.val;
+            } else {
+                t->scanfrom = tok.val;
+            }
+            return OK;
 
         case T_Back:
             DBufFree(&buf);
@@ -1065,15 +1282,14 @@ static int ParseScanFrom(ParsePtr s, Trigger *t, int type)
                 Eprint("%s: %s", word, GetErr(E_DAY_TWICE));
                 return E_DAY_TWICE;
             }
-            if (tok.val < 0) {
+            if (tok.val > 0) {
                 tok.val = -tok.val;
             }
-            FromDSE(DSEToday - tok.val, &y, &m, &d);
-            /* Don't purge reminders with a relative scanfrom */
+            t->scanfrom = tok.val;
             s->expr_happened = 1;
             nonconst_debug(s->nonconst_expr, tr("Relative SCANFROM counts as a non-constant expression"));
             s->nonconst_expr = 1;
-            break;
+            return OK;
 
         default:
             if (tok.type == T_Illegal && tok.val < 0) {
@@ -1090,14 +1306,10 @@ static int ParseScanFrom(ParsePtr s, Trigger *t, int type)
                 DBufFree(&buf);
                 return E_BAD_DATE;
             }
-            t->scanfrom = DSE(y, m, d);
             if (type == FROM_TYPE) {
-                t->from = t->scanfrom;
-                if (t->scanfrom < DSEToday) {
-                    t->scanfrom = DSEToday;
-                }
+                t->from = DSE(y, m, d);
             } else {
-                t->from = NO_DATE;
+                t->scanfrom = DSE(y, m, d);
             }
 
             PushToken(DBufValue(&buf), s);
@@ -1115,7 +1327,7 @@ static int ParseScanFrom(ParsePtr s, Trigger *t, int type)
 /*  Trigger the reminder if it's a RUN or MSG type.            */
 /*                                                             */
 /***************************************************************/
-int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig const *tim, int dse, int is_queued, DynamicBuffer *output)
+int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig const *tim, int dse, int is_queued, DynamicBuffer *output, int *rr, int *gg, int *bb)
 {
     int r, y, m, d;
     int adjusted_for_newline = 0;
@@ -1180,17 +1392,37 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig const *tim, int dse, int is
         DBufFree(&buf);
         if (r) return r;
         t->typ = MSG_TYPE;
+
+        if (red < 0 || green < 0 || blue < 0 || red > 255 || green > 255 || blue > 255) {
+            red = -1;
+            green = -1;
+            blue = -1;
+            t->passthru[0] = 0;
+        }
     }
 /* If it's a MSG-type reminder, and no -k option was used, issue the banner. */
     if ((t->typ == MSG_TYPE || t->typ == MSF_TYPE) 
         && !DidMsgReminder && !NextMode && !msg_command && !is_queued) {
+        DynamicBuffer buf2;
+        DBufInit(&buf2);
         DidMsgReminder = 1;
-        if (!DoSubstFromString(DBufValue(&Banner), &buf,
+        if (!DoSubstFromString(DBufValue(&Banner), &buf2,
                                DSEToday, NO_TIME) &&
-            DBufLen(&buf)) {
-            printf("%s\n", DBufValue(&buf));
+            DBufLen(&buf2)) {
+            if (!JSONMode) {
+                printf("%s\n", DBufValue(&buf2));
+            } else {
+                if (JSONLinesEmitted) {
+                    printf("},\n");
+                }
+                JSONLinesEmitted++;
+                printf("{\"banner\":\"");
+                remove_trailing_newlines(&buf2);
+                PrintJSONString(DBufValue(&buf2));
+                printf("\"");
+            }
         }
-        DBufFree(&buf);
+        DBufFree(&buf2);
     }
 
 /* If it's NextMode, process as a ADVANCE_MODE-type entry, and issue
@@ -1265,7 +1497,7 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig const *tim, int dse, int is
     }
 
     /* Correct colors */
-    if (UseVTColors) {
+    if (JSONMode || UseVTColors) {
         if (red == -1 && green == -1 && blue == -1) {
             if (DefaultColorR != -1 && DefaultColorG != -1 && DefaultColorB != -1) {
                 red = DefaultColorR;
@@ -1279,6 +1511,17 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig const *tim, int dse, int is
             if (green > 255) green = 255;
             if (blue > 255) blue = 255;
         }
+        if (rr) *rr = red;
+        if (gg) *gg = green;
+        if (bb) *bb = blue;
+
+        /* Don't ANSI-colorize JSON output! */
+        if (JSONMode) {
+            is_color = 0;
+            red = -1;
+            green = -1;
+            blue = -1;
+        }
     }
 
     /* Put the substituted string into the substitution buffer */
@@ -1430,13 +1673,41 @@ int TriggerReminder(ParsePtr p, Trigger *t, TimeTrig const *tim, int dse, int is
 int ShouldTriggerReminder(Trigger const *t, TimeTrig const *tim, int dse, int *err)
 {
     int r, omit;
+    int calmode = (DoSimpleCalendar || DoCalendar) ? 1 : 0;
+    if (HideCompletedTodos) calmode = 0;
+
     *err = 0;
 
     /* Handle the ONCE modifier in the reminder. */
     if (!IgnoreOnce && t->once !=NO_ONCE && GetOnceDate() == DSEToday)
         return 0;
 
-    if (dse < DSEToday) return 0;
+    /* TODOs are handled differently */
+    if (t->is_todo && !calmode) {
+        /* Do NOT trigger if TODO has been completed through today (or later) */
+        if (t->complete_through != NO_DATE && t->complete_through >= DSEToday && dse <= t->complete_through) {
+            return 0;
+        }
+        /* DO trigger if has not been completed through trigger date */
+        if (t->complete_through == NO_DATE || t->complete_through < dse) {
+            /* Trigger date is in the past - overdue,  Trigger unless we're
+               more than max_overdue days late */
+            if (dse < DSEToday) {
+                if (t->max_overdue >= 0) {
+                    if (dse + t->max_overdue < DSEToday) {
+                        return 0;
+                    }
+                }
+                return 1;
+            }
+            /* Trigger date in future - use normal Remind rules */
+        } else {
+            /* We're complete as of trigger date */
+            return 0;
+        }
+    } else {
+        if (dse < DSEToday) return 0;
+    }
 
     /* Don't trigger timed reminders if DontIssueAts is true, and if the
        reminder is for today */
@@ -1530,7 +1801,7 @@ int DoSatRemind(Trigger *trig, TimeTrig *tt, ParsePtr p)
     ensure_satnode_mentions_trigdate(sat_node);
 
     iter = 0;
-    start = trig->scanfrom;
+    start = get_scanfrom(trig);
     while (iter++ < MaxSatIter) {
         dse = ComputeTriggerNoAdjustDuration(start, trig, tt, &r, 1, 0);
         if (r) {
@@ -1569,7 +1840,7 @@ int DoSatRemind(Trigger *trig, TimeTrig *tt, ParsePtr p)
         }
         if ((v.type == INT_TYPE && v.v.val) ||
             (v.type == STR_TYPE && *v.v.str)) {
-            AdjustTriggerForDuration(trig->scanfrom, dse, trig, tt, 1);
+            AdjustTriggerForDuration(get_scanfrom(trig), dse, trig, tt, 1);
             if (DebugFlag & DB_PRTTRIG) {
                 int y, m, d;
                 FromDSE(LastTriggerDate, &y, &m, &d);
@@ -1712,6 +1983,32 @@ static int ShouldTriggerBasedOnWarn(Trigger const *t, int dse, int *err)
     int r, omit;
     Value v;
     int lastReturnVal = 0; /* Silence compiler warning */
+    int calmode = (DoSimpleCalendar || DoCalendar) ? 1 : 0;
+
+    /* TODOs are handled differently */
+    if (t->is_todo && !calmode) {
+        /* Do NOT trigger if TODO has been completed through today (or later) */
+        if (t->complete_through != NO_DATE && t->complete_through >= DSEToday) {
+            return 0;
+        }
+        /* DO trigger if has not been completed through trigger date */
+        if (t->complete_through == NO_DATE || t->complete_through < dse) {
+            /* Trigger date is in the past - overdue,  Trigger unless we're
+               more than max_overdue days late */
+            if (dse < DSEToday) {
+                if (t->max_overdue >= 0) {
+                    if (dse + t->max_overdue < DSEToday) {
+                        return 0;
+                    }
+                }
+                return 1;
+            }
+            /* Trigger date in future - use normal Remind rules */
+        } else {
+            /* We're complete as of trigger date */
+            return 0;
+        }
+    }
 
     /* If no proper function exists, barf... */
     if (UserFuncExists(t->warn) != 1) {
